/**************************************************************************
 * Module LessonLocationChange (llocchange.js)
 * Contiene clases y funciones relacionadas con la actualizacion de la
 * lesson_location.
 * Las funciones de este modulo solo se llaman en la pagina de lanzamiento 
 * por lo que no es necesario que esten en findAPI.
 *
 * @date 19/07/2004
 **************************************************************************/

var DEFAULT_MASTERY = 50;
 
/**************************************
 * Class Node
 **************************************/

/**
 * Node (Constructor)
 * @param String id. Identificador de contenidos del nodo.
 * @param String cod. Nuevo codigo de posicion del nodo.
 * @param String status. Estado del nodo: se modifica durante la 
 * actualizacion.
 * @param String mastery. Nota de corte en las actividades (incluye la 'a').
 * @param String acc. Accion a realizar sobre este nodo.
 */
 
function Node(id,cod,status,mastery,acc)
{
	this.id = id;
	this.cod = cod;
	if(typeof(status)=='undefined')
		this.status = 'n'
	else
		this.status = status;
	this.mastery = mastery;
	this.acc = acc;
	this.parent = null;
	this.firstChild = null;
	this.nextSibling = null;
}

/**
 * Node_depth
 * Devuelve la profundidad del nodo, siendo 0 la de la raiz.
 * @return int. Profundidad del nodo.
 */

function Node_depth()
{
	if(this.cod == 'R0')
		return 0
	else {
		var sCod;
		
		sCod = this.cod.split('_');
		return sCod.length;
	}
}

/**
 * Node_parentCod
 * Devuelve el cod que deberia tener su padre, aunque no comprueba que este
 * este en el arbol.
 * @return String. Cod del padre; el de R0 es la cadena vacia.
 */
 
function Node_parentCod()
{
	if(this.depth()<=0)
		return ''
	else if(this.depth()==1)
		return 'R0'
	else {
		var sCod;

		sCod = this.cod.split('_');
		//sCod.pop();
		sCod = sCod.slice(0,sCod.length-1);
		return 'b'+sCod.join('_').substr(1);
	}
}

/**
 * Node_recalcActivity
 * Recalcula el estado de una actividad usando la nueva estructura del 
 * curso.
 */

function Node_recalcActivity()
{
	if(this.status.substr(0,1)=='p'||this.status.substr(0,1)=='f') {
		var score, mastery;

		mastery = parseInt(this.mastery.substr(1));	// elimina la 'a'
		if(isNaN(mastery)) mastery = DEFAULT_MASTERY;	// por defecto
		score = parseInt(this.status.substr(1));
		if(isNaN(score)) score = 0;
		
		if(score>=mastery)
			this.status = 'p'+score
		else
			this.status = 'f'+score;
	}
}

/**
 * Node_recalcActivityBlock
 * Recalcula el estado de un bloque de actividad usando la nueva estructura 
 * del curso.
 */

function Node_recalcActivityBlock()
{
	var score, count, mastery;
	var oneVisited, allComplete;
	var node;

	// extraemos la nota de corte para este bloque
	mastery = parseInt(this.mastery.substr(1));	// elimina la 'a'
	if(isNaN(mastery)) mastery = DEFAULT_MASTERY;	// por defecto
	
	// recorremos los hijos calculando la nota total y verificando si
	// todos estan visitados y completados
	score = 0; count = 0;
	allComplete = true; oneVisited = false;
	node = this.firstChild;
	while(node!=null && (allComplete || !oneVisited)) {
		switch(node.status.substr(0,1)) {
		case 'p':
		case 'f': 
			score += parseInt(node.status.substr(1));
			count++;
			oneVisited = true;
			break;
		case 'c':
			oneVisited = true;
			break;
		case 'n':
			allComplete = false;
			break;
		case 'i':
			oneVisited = true;
			allComplete = false;
			break;
		}
		node = node.nextSibling;
	}
	if(!oneVisited) {
		if(!allComplete)
			this.status = 'n';
		// lo contrario se da cuando no tiene hijos: se matiene el estado
	} else {
	 	if(allComplete) {
			if(count!=0)
				score = Math.round(score/count)
			else
				score = 0;
			if(score>=mastery)
				this.status = 'p'+score
			else
				this.status = 'f'+score;
		} else
			this.status = 'i';
	}
}
		
/**
 * Node_recalcTheoryBlock
 * Recalcula el estado de un bloque de teoria usando la nueva estructura 
 * del curso.
 */

function Node_recalcTheoryBlock()
{
	var allComplete, oneVisited;
	var node;

	// recorremos los hijos verificando si estan visitados y completados
	allComplete = true; oneVisited = false;
	node = this.firstChild;
	while(node!=null && (allComplete || !oneVisited)) {
		switch(node.status.substr(0,1)) {
		case 'c':
			oneVisited = true;
			break;
		case 'n':
			allComplete = false;
			break;
		case 'i':
			oneVisited = true;
			allComplete = false;
			break;
		}
		node = node.nextSibling;
	}
	if(!oneVisited) {
		if(!allComplete)
			this.status = 'n';
	} else {
		if(allComplete)
			this.status = 'c'
		else
			this.status = 'i';
	}
}
		
/**
 * Node_recalcStatus
 * Recalcula el estado del nodo usando la nueva estructura del curso.
 */

function Node_recalcStatus()
{
	switch(this.cod.substr(0,1)) {
	case 'p':	// pagina
		break;
	case 'a':	// actividad
		this.recalcActivity();
		break;
	default:	// bloque
		if(this.mastery.substr(0,1)=='a')	// bloque de actividades
			this.recalcActivityBlock()
		else								// bloque de teoria
			this.recalcTheoryBlock();
	}
}

new Node();
Node.prototype.depth = Node_depth;
Node.prototype.parentCod = Node_parentCod;
Node.prototype.recalcActivity = Node_recalcActivity;
Node.prototype.recalcActivityBlock = Node_recalcActivityBlock;
Node.prototype.recalcTheoryBlock = Node_recalcTheoryBlock;
Node.prototype.recalcStatus = Node_recalcStatus;

/**************************************
 * Class NodeQ
 *************************************/

/**
 * NodeQ (Constructor)
 */
 
function NodeQ()
{
	this.Q = new Array();
}

/**
 * NodeQ_length
 * Devuelve el tamano de la cola.
 * @return int. Tamano de la cola.
 */

function NodeQ_length()
{
	return this.Q.length;
}

/**
 * NodeQ_shift
 * Devuelve y elimina el primer elemento de la cola.
 * @return Node. Primer nodo de la cola.
 */

function NodeQ_shift()
{
	//return this.Q.shift();
    var r = this.Q[0];
    this.Q = this.Q.slice(1);
    return r;
}

/**
 * NodeQ_insertDepthPrior
 * Inserta contenidos en la cola con prioridad para los nodos mas profundos.
 * No admite repetidos.
 * @param Node node. Nodo a insertar.
 */
 
function NodeQ_insertDepthPrior(node)
{
	if(this.Q.length == 0) this.Q = this.Q.concat(node); //this.Q.splice(0,0,node)
	else {
		var exists, i;
		
		exists = false;
		for(i=0;i<this.Q.length && !exists 
			&& node.depth()<=this.Q[i].depth();i++) {
			if(this.Q[i].cod==node.cod) exists=true;
		}
		if(!exists) {
		    //this.Q.splice(i,0,node);
		    var a = this.Q.slice(0,i);
		    a = a.concat(node);
		    this.Q = a.concat(this.Q.slice(i));
		}
	}
}

new NodeQ();
NodeQ.prototype.length = NodeQ_length;
NodeQ.prototype.shift = NodeQ_shift;
NodeQ.prototype.insertDepthPrior = NodeQ_insertDepthPrior;

/**************************************
 * Class CourseTree
 *************************************/

/**
 * CourseTree (Constructor)
 * Crea el arbol de la nueva estructura del curso.
 * @param oldLL. Antigua cadena de navegacion con sus estados.
 * @param oldCS. Antigua cadena de contenidos para relacionar los id.
 * @param newCV. Nueva estructura del curso.
 * @param newCS. Nueva cadena de contenidos.
 */

function CourseTree(oldLL,oldCS,newCV,newCS)
{
	// desmenuzamos las cadenas antiguas
	var OLL, OCS, oldStat = new Array();

 	OLL = oldLL.split(';'); 
 	//OLL.pop();		// elimina el ';.'
 	OLL = OLL.slice(0,OLL.length-1);
 	OCS = oldCS.split(';');
 	for(var i=0;i<OCS.length;i++)
 		oldStat['e'+OCS[i].split(',')[0]] = OLL[i];
 	
 	// a partir del newCV creamos el arbol
	this.T = new Array();
 	var NCS, NCV;
 	var last;
 	
 	NCS = newCS.split(';');
 	NCV = newCV.split(';'); 
 	//NCV.pop();		// elimina el ';.'
 	NCV = NCV.slice(0,NCV.length-1);

 	last = null;
 	for(var i=0;i<NCV.length;i++) {
 		var node;
	 	var pCod, pos, con;
 		
 		pos = NCV[i].split(',');
 		con = NCS[i].split(',');
 		node = new Node(con[0],pos[0],oldStat['e'+con[0]],pos[1],con[1]);
 		this.T[i] = node;
 		if(i>0) {	// descartamos la raiz
 			pCod = node.parentCod();
 			// mientras el ultimo nodo insertado no sea hermano ni padre
 			// ascendemos
 			while(pCod!=last.parentCod() && pCod!=last.cod) 
 				last = last.parent;
 			// si son hermanos
 			if(pCod==last.parentCod()) {
 				node.parent = last.parent;
 				last.nextSibling = node;
 			// si es el padre
 			} else if(pCod==last.cod) {
 				node.parent = last;
 				last.firstChild = node;
 			}
 		}
		last = node;
 	}
}

/**
 * CourseTree_findNode
 * Devuelve el nodo que tiene un determinado cod.
 * @param String cod. cod del nodo buscado.
 * @return Node. Nodo buscado; null si no lo encuentra.
 */
 
function CourseTree_findNode(cod)
{
	for(var i=0;i<this.T.length;i++)
		if(this.T[i].cod == cod)
			return this.T[i];
	return null;
}

/**
 * CourseTree_redoLLoc
 * Rehace la cadena de navegacion a partir de los estados en el arbol.
 * @return String. La cadena de navegacion.
 */

function CourseTree_redoLLoc()
{
	var lloc='';
	
	for(var i=0;i<this.T.length;i++) 
		lloc += this.T[i].status + ';';
	lloc += '.';
	return lloc;
}

/**
 * CourseTree_llocChange
 * Procesa el cambio de la cadena de navegacion entre versiones de cursos.
 * @return String. La nueva cadena de navegacion.
 */
 
function CourseTree_llocChange() {
	var nodeQ = new NodeQ();	// lista para los que hay que recalcular
	
	// recorremos la estructura del nuevo curso ejecutando las acciones
	// indicadas sobre los nodos
	for(var i=0;i<this.T.length;i++) {
		switch(this.T[i].cod.substr(0,1)){
		case 'p':		
		case 'a':	// pagina o actividad
			// si no tiene accion explicita, hereda
			if(this.T[i].acc=='') 
				this.T[i].acc = this.T[i].parent.acc;

			switch(this.T[i].acc) {
			case 'i':	// reinicia: lo pone a 'n'
				this.T[i].status = 'n';
				break;
			case '':
			case 'c':	// recalcula: e inserta su padre en la cola
				this.T[i].recalcStatus();
				// inserta el padre para recalcularlo
				nodeQ.insertDepthPrior(this.T[i].parent);
				break;
			case 'n':	// nada: conserva el estado
				break;
			}
			break;
		default:	// bloque
			// si no tiene accion explicita, hereda
			if(this.T[i].acc=='' && this.T[i].parent!=null) 
				this.T[i].acc = this.T[i].parent.acc;

			switch(this.T[i].acc){
			case 'i':	// reinicia: lo transmite a todos sus hijos
				var node;
				
				this.T[i].status = 'n';
				node = this.T[i].firstChild;
				while(node!=null) {
					node.acc = 'i';
					node = node.nextSibling;
				}
				break;
			case '':
			case 'c':	// recalcula: metemos en la cola de calculo
				nodeQ.insertDepthPrior(this.T[i]);
				break;
			case 'n':	// nada: conserva el estado
				break;
			}
		}
	}

	// recalcula los bloques de la lista de calculo
	while(nodeQ.length() > 0) {
		var node;
		
		node = nodeQ.shift();
		node.recalcStatus();
		if(node.parent != null)
			nodeQ.insertDepthPrior(node.parent);
	}

	// rehacemos la nueva lesson_location
	return this.redoLLoc();
}

new CourseTree('','','','');
CourseTree.prototype.findNode = CourseTree_findNode;
CourseTree.prototype.redoLLoc = CourseTree_redoLLoc;
CourseTree.prototype.llocChange = CourseTree_llocChange;

/**************************************
 * Funciones generales
 **************************************/

/**
 * ActualizaCoreVendor
 * Comprueba las versiones y, si no coinciden, lanza el cambio.
 */

function ActualizaCoreVendor(nuevoalumno)
{
	ver=nexeapi.NEXGetHashParameter("ver");
	if(ver==null) ver = '0';

	if(ver!=core_version && !nuevoalumno) {
		var tree = new CourseTree(API.LMSGetValue(AICC_Protocol),
			eval('contents_string_'+ver), core_vendor, contents_string);

		API.LMSSetValue(AICC_Protocol,tree.llocChange());
		
		nexeapi.NEXSetHashParameter("ver",core_version);
		alert('ADVERTENCIA\n'+
			  'Esta accion formativa ha cambiado, y su estado\n'+
			  'ha sido actualizado.');
	} else {
		nexeapi.NEXSetHashParameter("ver",core_version);
	}
}

function IncluyeHistorico()
{
	ver=nexeapi.NEXGetHashParameter("ver");
	if(ver==null) ver = '0';

	if(ver!=core_version) {
		document.write('<s'+'cript language="javascript" src="history/contents-'
			+ver+'.js"></s'+'cript>');
		document.write('<s'+'cript language="javascript" src="history/core-'
			+ver+'.js"></s'+'cript>');
	}
}
	